import 'dart:async';
import 'dart:ui' as ui;

import 'package:flutter/material.dart';
import 'package:flutter/foundation.dart';
import '../ui.dart';

class SplashScreen extends StatefulWidget {
  const SplashScreen({
    @required this.splash,
    @required this.child,
    this.duration = const Duration(seconds: 1),
    Key key,
  }) : super(key: key);

  final Widget splash;
  final Widget child;
  final Duration duration;

  @override
  _SplashScreenState createState() => _SplashScreenState();

  @override
  void debugFillProperties(DiagnosticPropertiesBuilder properties) {
    super.debugFillProperties(properties);
    properties
      ..add(DiagnosticsProperty<Duration>('duration', duration))
      ..add(DiagnosticsProperty<Widget>('splash', splash))
      ..add(DiagnosticsProperty<Widget>('child', child));
  }
}

class _SplashScreenState extends State<SplashScreen> {
  bool _showSplash = true;

  StreamSubscription<bool> _subscription;

  @override
  void initState() {
    _subscription ??= UI().splashStream.listen((showSplash) {
      _subscription.pause(Future.delayed(widget.duration));
      setState(() => _showSplash = showSplash);
    });
    super.initState();
  }

  @override
  void dispose() {
    _subscription?.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      textDirection: ui.TextDirection.ltr,
      children: <Widget>[
        widget.child,
        IgnorePointer(
          ignoring: !_showSplash,
          child: AnimatedOpacity(
            curve: Curves.easeOutQuad,
            opacity: _showSplash ? 1 : 0,
            duration: widget.duration,
            child: widget.splash,
          ),
        ),
      ],
    );
  }
}
